/*
 * This file is part of libipc (https://github.com/TheCrypt0/libipc).
 * Copyright (c) 2019 roleo.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <errno.h>
#include <limits.h>
#include <mqueue.h>

#define IPC_QUEUE_NAME          "/ipc_dispatch"
#define IPC_MESSAGE_MAX_SIZE    512

#define NONE -1

#define IPC_MOTION_START "\x01\x00\x00\x00\x02\x00\x00\x00\x7c\x00\x7c\x00\x00\x00\x00\x00"
#define IPC_MOTION_STOP "\x01\x00\x00\x00\x02\x00\x00\x00\x7d\x00\x7d\x00\x00\x00\x00\x00"

#define IPC_SWITCH_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x75\x00\x01\x00\x00\x00\x00\x00"
#define IPC_SWITCH_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x74\x00\x01\x00\x00\x00\x00\x00"

#define SWITCH_OFF 0
#define SWITCH_ON  1

#define IPC_SENS_HIGH   "\x01\x00\x00\x00\x08\x00\x00\x00\x27\x10\x01\x00\x04\x00\x00\x00\x00\x00\x00\x00"
#define IPC_SENS_MEDIUM "\x01\x00\x00\x00\x08\x00\x00\x00\x27\x10\x01\x00\x04\x00\x00\x00\x01\x00\x00\x00"
#define IPC_SENS_LOW    "\x01\x00\x00\x00\x08\x00\x00\x00\x27\x10\x01\x00\x04\x00\x00\x00\x02\x00\x00\x00"

#define SENSITIVITY_HIGH   0
#define SENSITIVITY_MEDIUM 1
#define SENSITIVITY_LOW    2

#define IPC_LED_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x77\x00\x01\x00\x00\x00\x00\x00"
#define IPC_LED_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x76\x00\x01\x00\x00\x00\x00\x00"

#define LED_OFF 0
#define LED_ON  1

#define IPC_SAVE_ALWAYS "\x02\x00\x00\x00\x08\x00\x00\x00\x79\x00\x01\x00\x00\x00\x00\x00"
#define IPC_SAVE_DETECT "\x02\x00\x00\x00\x08\x00\x00\x00\x78\x00\x01\x00\x00\x00\x00\x00"

#define SAVE_ALWAYS 0
#define SAVE_DETECT 1

#define IPC_IR_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x24\x10\x01\x00\x04\x00\x00\x00\x02\x00\x00\x00"
#define IPC_IR_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x24\x10\x01\x00\x04\x00\x00\x00\x01\x00\x00\x00"

#define IR_OFF 0
#define IR_ON  1

#define IPC_ROTATE_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x7b\x00\x01\x00\x00\x00\x00\x00"
#define IPC_ROTATE_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x7a\x00\x01\x00\x00\x00\x00\x00"

#define ROTATE_OFF 0
#define ROTATE_ON  1

#define IPC_MOVE_RIGHT "\x01\x00\x00\x00\x08\x00\x00\x00\x06\x40\x06\x40\x18\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00"
#define IPC_MOVE_LEFT  "\x01\x00\x00\x00\x08\x00\x00\x00\x06\x40\x06\x40\x18\x00\x00\x00\x03\x00\x00\x00\x00\x00\x00\x00"
#define IPC_MOVE_DOWN  "\x01\x00\x00\x00\x08\x00\x00\x00\x06\x40\x06\x40\x18\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00"
#define IPC_MOVE_UP    "\x01\x00\x00\x00\x08\x00\x00\x00\x06\x40\x06\x40\x18\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00"
#define IPC_MOVE_STOP  "\x01\x00\x00\x00\x08\x00\x00\x00\x07\x40\x01\x00\x00\x00\x00\x00"

#define MOVE_RIGHT 0
#define MOVE_LEFT  1
#define MOVE_DOWN  2
#define MOVE_UP    3
#define MOVE_STOP  4

#define IPC_GOTO_PRESET "\x01\x00\x00\x00\x08\x00\x00\x00\x02\x40\x01\x00\x04\x00\x00\x00\xff\x00\x00\x00"

#define GOTO_PRESET 0

#define IPC_XXX_0 "\x01\x00\x00\x00\x04\x00\x00\x00\x71\x00\x71\x00\x00\x00\x00\x00\xa1\x0e\x9a\x5e"

#define IPC_AI_HUMAN_DETECTION_OFF "\x01\x00\x00\x00\x08\x00\x00\x00\x3c\x10\x01\x00\x04\x00\x00\x00\x00\x00\x00\x00"
#define IPC_AI_HUMAN_DETECTION_ON  "\x01\x00\x00\x00\x08\x00\x00\x00\x3c\x10\x01\x00\x04\x00\x00\x00\x01\x00\x00\x00"

#define AI_HUMAN_DETECTION_OFF 0
#define AI_HUMAN_DETECTION_ON  1

#define IPC_BABYCRYING_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x29\x10\x01\x00\x04\x00\x00\x00\x01\x00\x00\x00"
#define IPC_BABYCRYING_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x29\x10\x01\x00\x04\x00\x00\x00\x02\x00\x00\x00"

#define BABYCRYING_OFF 0
#define BABYCRYING_ON  1

#define IPC_SOUND_DETECTION_OFF "\x02\x00\x00\x00\x08\x00\x00\x00\x3a\x10\x01\x00\x04\x00\x00\x00\x01\x00\x00\x00"
#define IPC_SOUND_DETECTION_ON  "\x02\x00\x00\x00\x08\x00\x00\x00\x3a\x10\x01\x00\x04\x00\x00\x00\x02\x00\x00\x00"

#define SOUND_DETECTION_OFF 0
#define SOUND_DETECTION_ON  1

#define IPC_SOUND_SENS_50 "\x02\x00\x00\x00\x08\x00\x00\x00\x3b\x10\x01\x00\x04\x00\x00\x00\x32\x00\x00\x00"
#define IPC_SOUND_SENS_60 "\x02\x00\x00\x00\x08\x00\x00\x00\x3b\x10\x01\x00\x04\x00\x00\x00\x3c\x00\x00\x00"
#define IPC_SOUND_SENS_70 "\x02\x00\x00\x00\x08\x00\x00\x00\x3b\x10\x01\x00\x04\x00\x00\x00\x46\x00\x00\x00"
#define IPC_SOUND_SENS_80 "\x02\x00\x00\x00\x08\x00\x00\x00\x3b\x10\x01\x00\x04\x00\x00\x00\x50\x00\x00\x00"
#define IPC_SOUND_SENS_90 "\x02\x00\x00\x00\x08\x00\x00\x00\x3b\x10\x01\x00\x04\x00\x00\x00\x5a\x00\x00\x00"

#define SOUND_SENS_50 50
#define SOUND_SENS_60 60
#define SOUND_SENS_70 70
#define SOUND_SENS_80 80
#define SOUND_SENS_90 90
